/*
 * Copyright (c) 2005, 2009 Sun Microsystems, Inc. All Rights Reserved.
 * Use is subject to license terms.
 */
/* 
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Basant Kukreja <basantk@php.net>                             |
   +----------------------------------------------------------------------+
 */

#ifndef _ZEND_LOCK_H
#define _ZEND_LOCK_H

#ifndef PHP_WIN32
#if defined(PTHREADS)
typedef pthread_rwlock_t zend_rwlock_t;
#else
#error pthread support is required for zend locks
#endif
#else
#if _WIN32_WINNT >= 0x0600
#define WIN_HAVE_SRW_LOCK 1
#endif

#ifdef WIN_HAVE_SRW_LOCK
typedef SRWLOCK zend_rwlock_t;
#else
typedef CRITICAL_SECTION zend_rwlock_t;
#endif
#endif

static inline void zend_init_rwlock(zend_rwlock_t* lock)
{
#ifdef PHP_WIN32
#ifdef WIN_HAVE_SRW_LOCK
	InitializeSRWLock(lock);
#else
	InitializeCriticalSection(lock);
#endif
#else
	pthread_rwlock_t initlock = PTHREAD_RWLOCK_INITIALIZER;
	memcpy(lock, &initlock, sizeof(initlock));
	pthread_rwlock_init(lock, NULL);
#endif
}

static inline void zend_acquire_read_lock(zend_rwlock_t* lock)
{
#ifdef PHP_WIN32
#ifdef WIN_HAVE_SRW_LOCK
	AcquireSRWLockShared(lock);
#else
	EnterCriticalSection(lock);
#endif
#else
	pthread_rwlock_rdlock(lock);
#endif
}

static inline void zend_acquire_write_lock(zend_rwlock_t* lock)
{
#ifdef PHP_WIN32
#ifdef WIN_HAVE_SRW_LOCK
	AcquireSRWLockExclusive(lock);
#else
	EnterCriticalSection(lock);
#endif
#else
	pthread_rwlock_wrlock(lock);
#endif
}

static inline void zend_release_read_lock(zend_rwlock_t* lock)
{
#ifdef PHP_WIN32
#ifdef WIN_HAVE_SRW_LOCK
	ReleaseSRWLockShared(lock);
#else
	LeaveCriticalSection(lock);
#endif
#else
	pthread_rwlock_unlock(lock);
#endif
}

static inline void zend_release_write_lock(zend_rwlock_t* lock)
{
#ifdef PHP_WIN32
#ifdef WIN_HAVE_SRW_LOCK
	ReleaseSRWLockExclusive(lock);
#else
	LeaveCriticalSection(lock);
#endif
#else
	pthread_rwlock_unlock(lock);
#endif
}

#endif
